// Code generated by go-swagger; DO NOT EDIT.

package container

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// UpdateResponse ContainerUpdateResponse
//
// Response for a successful container-update.
//
// swagger:model UpdateResponse
type UpdateResponse struct {

	// Warnings encountered when updating the container.
	// Example: ["Published ports are discarded when using host network mode"]
	Warnings []string `json:"Warnings"`
}
