// Code generated by protoc-gen-go-fieldpath. DO NOT EDIT.
// source: events/sandbox.proto
package events

// Field returns the value for the given fieldpath as a string, if defined.
// If the value is not defined, the second value will be false.
func (m *SandboxCreate) Field(fieldpath []string) (string, bool) {
	if len(fieldpath) == 0 {
		return "", false
	}
	switch fieldpath[0] {
	case "sandbox_id":
		return string(m.SandboxID), len(m.SandboxID) > 0
	}
	return "", false
}

// Field returns the value for the given fieldpath as a string, if defined.
// If the value is not defined, the second value will be false.
func (m *SandboxStart) Field(fieldpath []string) (string, bool) {
	if len(fieldpath) == 0 {
		return "", false
	}
	switch fieldpath[0] {
	case "sandbox_id":
		return string(m.SandboxID), len(m.SandboxID) > 0
	}
	return "", false
}

// Field returns the value for the given fieldpath as a string, if defined.
// If the value is not defined, the second value will be false.
func (m *SandboxExit) Field(fieldpath []string) (string, bool) {
	if len(fieldpath) == 0 {
		return "", false
	}
	switch fieldpath[0] {
	// unhandled: exit_status
	// unhandled: exited_at
	case "sandbox_id":
		return string(m.SandboxID), len(m.SandboxID) > 0
	}
	return "", false
}
