// Code generated by go-swagger; DO NOT EDIT.

package network

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the swagger generate command

// SubnetStatus subnet status
//
// swagger:model SubnetStatus
type SubnetStatus struct {

	// Number of IP addresses in the subnet that are in use or reserved and are therefore unavailable for allocation, saturating at 2<sup>64</sup> - 1.
	//
	IPsInUse uint64 `json:"IPsInUse"`

	// Number of IP addresses within the network's IPRange for the subnet that are available for allocation, saturating at 2<sup>64</sup> - 1.
	//
	DynamicIPsAvailable uint64 `json:"DynamicIPsAvailable"`
}
